# gypsy_models.R
#
# Jari Vauhkonen
# 4 July 2018
#
# This file contains implementations of three GYPSY sub-models:
# - 4. Top height model
# - 6. Basal area increment model (non-spatial)
# - 10. Gross total volume model
#
# The implementations are based on Huang et al. (2009) "A Growth and Yield Projection System (GYPSY) for Natural and Post-harvest Stands in Alberta",
# downloaded from https://www1.agric.gov.ab.ca/$department/deptdocs.nsf/all/formain15784/$file/GYPSY-Natural-PostHarvestStands-Alberta-May21-2009.pdf?OpenElement
# on 10 March 2018 (Created: 21.5.2009 and Modified: 5.8.2010 according to the document properties of the PDF).
# For ease, the document GYPSY_model_documentation.pdf is included in the same directory with this script file.
# 

# Top height model (p. 5 in GYPSY_model_documentation.pdf)

Htop <- function(species, SI, AGE) {
   param = list( aw = c(9.908888, -3.92451, -0.32778, 0.134376),
                 sb = c(14.56236, -6.04705, -1.53715, 0.240174),
                 pl = c(12.84571, -5.73936, -0.91312, 0.150668),
                 sw = c(12.14943, -3.77051, -0.28534, 0.165483) )

   param <- param[species][[1]]

   if (species=='aw') {
     AGE = AGE + 4
     htop = SI * ((1 + exp(param[1] + param[2] * sqrt(log(1+50)) + param[3] * (log(SI))^2 + param[4] * sqrt(50))) / (1 + exp(param[1] + param[2] * sqrt(log(1+AGE)) + param[3] * (log(SI))^2 + param[4] * sqrt(50))))
   } else if (species=='sw') {
     AGE = AGE + 12
     htop = SI * ((1 + exp(param[1] + param[2] * sqrt(log(1+50^2)) + param[3] * (log(SI))^2 + param[4] * sqrt(50))) / (1 + exp(param[1] + param[2] * sqrt(log(1+AGE^2)) + param[3] * (log(SI))^2 + param[4] * sqrt(50))))
   } else {
     if (species=='sb') {
        AGE = AGE + 15
     } else {
        AGE = AGE + 8
     }
     htop = SI * ((1 + exp(param[1] + param[2] * sqrt(log(1+50)) + param[3] * log(SI) + param[4] * sqrt(50))) / (1 + exp(param[1] + param[2] * sqrt(log(1+AGE)) + param[3] * log(SI) + param[4] * sqrt(50))))
   }

   htop
}


# Basal area increment model, non-spatial (pp. 8-9 in GYPSY_model_documentation.pdf)

# AGE = bhage

BAINC <- function(species, SI, AGE, Nini, BA, SC=1, k=0) {
   param = list( aw = c(0.751313, 0.018847, 1.143762, -0.03475, 0.835189),
                 sb = c(0.966285, 0.056315, 0.17191),
                 pl = c(3.923984, 0.05752, 0.560402, 0.672506, -0.00358, 0.775765),
                 sw = c(0.089153, 0.072171, -0.11483, 5.839408, 1.753002, 0.239521) )

   param <- param[species][[1]]

   if (species=='aw') {
      bainc = (10^-4 * param[1] * AGE^2 * exp(-1*param[2]*AGE^(0.5+param[1])) * (log(1 + Nini * sqrt(1+AGE)))^2 * SI * SC^param[5]) / ( (1+BA)^param[3] * (1+exp(1-(log(1+SC^2)/2))) )# + (param[4] * log(0.01 + AGE/10))
   } else if (species=='sb') {
      bainc = 10^-4 * param[1] * exp(-1*param[2]*AGE) * AGE^(sqrt(param[1])+param[2]) * SC^param[3] * (  ((1 + sqrt(Nini) * sqrt(1+AGE)) * log(1+SI) * exp(-(Nini/4)/10000)) / (1+BA)^param[2] )
   } else if (species=='sw') {
      bainc = (10^-4 * param[1] * ((param[2]+AGE)^2) * ((1+AGE)^(sqrt(param[1])+param[2]-param[3])) * exp(-1*param[2]*AGE) * (SC^param[6]) * ( (log(1 + Nini * sqrt(1+AGE)))^2 * sqrt(SI) * exp(-1 * ((Nini/10)/10000)) ) )   /   ( 1+ exp(1 + k + (log(1+ sqrt(Nini)/10000)/2) + param[3] * log(1+BA)) )
   } else {
      bainc = ( 10^-4 * param[1] * AGE * exp(-1*param[2]*AGE) * (1 + log(1+AGE)/2) * ((1 + param[3] + SI^param[6]) * sqrt(Nini) * exp(-(1/3) * (Nini/10000))) ) / (1 + exp(k/2 + log(1+(1/3)*(Nini/10000))-param[3]*sqrt(SC) + param[4]* log(1+BA))) + (param[5] * log(0.01+AGE/10))
   }

   bainc
}


# Gross total volume model (p. 15 in GYPSY_model_documentation.pdf)

Tvol <- function(species, BA, H) {

   param = list( aw = c(0.24872, 0.98568, 0.85728, -24.9961),
                 sb = c(0.48628, 0.98296, 0.91060),
                 pl = c(0.19409, 0.98828, 0.94935, -3.39036),
                 sw = c(0.41104, 0.98311, 0.97106) )

   param <- param[species][[1]]

   if (species=='sw' | species=='sb') {
      tvol = param[1] * BA^param[2] * H^param[3]
   } else {
      tvol = param[1] * BA^param[2] * H^param[3] * exp(1+param[4]/(H^2+1))
   }

   tvol

}


