# matching.R
#
# Jari Vauhkonen
# 4 July 2018
#
# This script first loads the GYPSY models and prepares the template database according to them.
# The script then matches observations given below with the templates.
#
# Note: results with two distinct databases with the different range for Nini parameter were obtained by changing these values hard-coded in generate_templates.R
#

rm(list=ls())
setwd("C:/temp/")

# Load GYPSY models
source("gypsy_models.R")

# Prepare the template database using ranges of input values defined in the script below
source("generate_templates.R")

# A vector of length 4 - mean ages for 4 species groups AW, PL, SB, and SW, respectively (values from Table 1 of Tompalski et al.)
a <- c(81,112,117,120)

# Three vectors of length 4 - mean height (h), basal area (g), and volume (v) for plots measured in Time T1. The same species groups and in same order as above (values from Table 2 of Tompalski et al.)
h.t1 <- c(19.6,12.9,15.4,21.8)
g.t1 <- c(32.3,22.8,30.6,36.1)
v.t1 <- c(269.3,162.5,190.4,304.4)

# A support vector listing the attributes of interest - mean height (h), basal area (g), and volume (v)
attrs <- c('h','g','v')

# Loop over the number of dominant species
for (sp in 1:length(dominant.species)) {
  inds.t1 <- c()

  # Loop over the number of attributes of interest
  for (attr in 1:3) {
    # Select the candidate templates relevant for this species
    candidate.curves <- list( h=sp.h.curves[sp], g=sp.g.curves[sp], v=sp.v.curves[sp] )
    # Prepare the test case to match the curves with - a list including the mean stand attributes for this species
    current.means.t1 <- list( a=a[sp], h=h.t1[sp], g=g.t1[sp], v=v.t1[sp] )
    # Look at projected values only in the time point of matching (= the current stand age stored in current.means.t1$a); sorry about the clumsy convention [[1]][1][[1]] to "un-list list values"...
    candidates.t1 <- as.numeric(candidate.curves[attr][[1]][1][[1]][,current.means.t1$a])
    # Compute the minimum difference between the current case attribute and the candidates; Store the index of the curve with the minimum difference
    inds.t1 <- c(inds.t1, sort(abs( current.means.t1[attr+1][[1]] - candidates.t1),ind=T)$ix[1])
  } # end loop attr;
    # As a result, you have a sequence of three indices of the curves with minimum difference to the three attributes of interest of this species

  # Store the indexes of the closest curves with species-specific indicator
  if (sp==1) {
    inds.sp.t1 <- c(dominant.species[sp], inds.t1)
  } else {
    inds.sp.t1 <- rbind(inds.sp.t1, c(dominant.species[sp], inds.t1))
  }
} # end loop dominant species;
# As a result, inds.sp.t1 contains a species indicator and three indexes pointing to closest templates for the three forest attributes in the template database.
# Note that the maximum index value and therefore the maximum number of possible templates differs per attribute: it can be obtained as
# - length(site.index) for Htop, i.e. there is as many possible height sequences as site index values
# - length(site.index) * length(seq.for.nini) for basal area, i.e. number of possible site indexes * number of possible initial density (Nini) values
# - length(site.index) * length(seq.for.nini) also for volume, because volume curves are obtained using Htop and Basal_area as its arguments

# How to compute "mean curves" from such values is particularly unclear based on the paper of Tompalski et al.;
# However, it could be obtained as means of SI and Nini.
# To fetch the site index and Nini parameters used for computing the specific template curve selected above, 
# one must look at vectors nini.values and si.values that were continuously added with the values of Nini and SI during the initialization (see initialize.R).
# Loop over the number of dominant species
si.sps <- nini.sps <- c()
for (sp in 1:length(dominant.species)) {
  # Compute a mean of site indexes used for height and site indexes used for basal area & volume based on the above lists of indices...
  si.w1 <- mean(c(site.index[as.numeric(inds.sp.t1[sp,2])],  si.values[as.numeric(inds.sp.t1[sp,3:4])]))
  si.ind1 <- sort(abs(site.index - si.w1),ind=T)$ix[1] #round(mean(c(site.index[as.numeric(inds.sp.t1[sp,2])],  si.values[as.numeric(inds.sp.t1[sp,3:4])])),0) #  si.sps <- c(si.sps, which(site.index==si.w1))

  # Compute a mean of Ninis used to compute basal area and volume...
  nini.w1 <- seq.for.nini[ sort(abs(seq.for.nini-mean(nini.values[as.numeric(inds.sp.t1[sp,3:4])])),ind=T)$ix[1] ]

  # Store the index values - differently for Htop vs. the other attributes (see above); sorry for clumsy naming conventions.
  si.sps <- c(si.sps, si.ind1)
  nini.sps <- c(nini.sps, which(si.values==site.index[si.ind1] & nini.values==nini.w1))
}
 
